#include "nvse/GameTiles.h"
#include "nvse/Utilities.h"
#include "nvse/GameAPI.h"
#include <string>

const _TraitNameToID TraitNameToID = (_TraitNameToID)0x009F97B0;

UInt32 Tile::TraitNameToID(const char * traitName)
{
	return ::TraitNameToID(traitName);
}

Tile::Value * Tile::GetValue(UInt32 typeID)
{
	// values are sorted so could use binary search but these are not paricularly large arrays
	for(UInt32 i = 0; i < values.size; i++)
	{
		Tile::Value * val = values[i];
		if(val && val->id == typeID)
			return val;
	}

	return NULL;
}

Tile::Value * Tile::GetValue(const char * valueName)
{
	UInt32 typeID = TraitNameToID(valueName);

	return GetValue(typeID);
}

Tile * Tile::GetChild(const char * childName)
{
	Tile * child = NULL;

	for(tList<ChildNode>::Iterator iter = childList.Begin(); !iter.End(); ++iter)
	{
		if(*iter && iter->child && !_stricmp(iter->child->name.m_data, childName))
		{
			child = iter->child;
			break;
		}
	}

	return child;
}

Tile::Value * Tile::GetComponentValue(const char * componentPath)
{
	Tokenizer tokens(componentPath, "\\/");
	std::string curToken;
	Tile * parentTile = this;

	while(tokens.NextToken(curToken) != -1 && parentTile)
	{
		DEBUG_PRINT("childName: %s", curToken.c_str());

		Tile * child = parentTile->GetChild(curToken.c_str());
		if(!child)
		{
			// didn't find child; is this last token?
			if(tokens.NextToken(curToken) == -1)
				return parentTile->GetValue(curToken.c_str());
			else	// nope, error
				return NULL;
		}
		else
			parentTile = child;
	}

	// couldn't locate one of the child nodes
	return NULL;
}

std::string Tile::GetQualifiedName(void)
{
	std::string qualifiedName;

	//if(parent && !parent->GetFloatValue(kTileValue_class, &parentClass))	// i.e., parent is not a menu
	if(parent)
		qualifiedName = parent->GetQualifiedName() + "\\";

	qualifiedName += name.m_data;

	return qualifiedName;
}

void Tile::Dump(void)
{
	_MESSAGE("%s", name.m_data);
	gLog.Indent();

	_MESSAGE("values:");

	gLog.Indent();
	
	for(UInt32 i = 0; i < values.size; i++)
	{
		Value	* val = values[i];

		if(val->str)
			_MESSAGE("%08X: %s", val->id, val->str);
		else if(val->action)
			_MESSAGE("%08X: action %08X", val->id, val->action);
		else
			_MESSAGE("%08X: %f", val->id, val->num);
	}

	gLog.Outdent();

	for(tList <ChildNode>::Iterator iter = childList.Begin(); !iter.End(); ++iter)
	{
		ChildNode	* node = iter.Get();
		if(node)
		{
			Tile	* child = node->child;
			if(child)
			{
				child->Dump();
			}
		}
	}

	gLog.Outdent();
}
